package ucot.input;

import java.io.IOException;
import java.net.URL;
import java.util.Properties;


public interface InputInterface {
	/**
	 * Parses given usecase. Stores Parsed data to given ParsedData object.
	 * @param url Url where input is loaded from
	 * @return Steps read from the input
	 */
	UseCaseCollection read(URL url) throws IOException;

	/**
	 * Test if this input can read file specified in url.
	 * @param url url of the file to try
	 * @return true if input can read the file
	 */
	boolean canRead(URL url);
	
	/**
	 * Sets options for the adapter
	 * @param options options for the adapter
	 */
	void setOptions(Properties options);
	
	/**
	 * Returns adapter's options
	 * @return adapter's options
	 */
	Properties getOptions();
	
}


