package ucot.input;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.URL;
import java.util.Properties;



public class SimpleInputAdapter implements InputInterface {
	private Properties options = new Properties();	
	
	public UseCaseCollection read(URL url) throws IOException {
		UseCaseCollection collection = new UseCaseCollection();
		UseCase usecase = new UseCase();		
				
		BufferedReader reader = new BufferedReader(new InputStreamReader(url.openStream()));
		boolean inside = false; // Are we inside usecase.
		boolean insideSteps = false; 
		boolean nextIsName = false;		

		
		while(reader.ready()){
			//Read a line
			String line = reader.readLine().trim();
			
			//If next should be the usecase name
			if (nextIsName && line.length()>0){
				usecase.setName(line);				
				nextIsName = false;
			}
			
			//or hasn't the usecase even started yet
			if (!inside){
				//If there is [name], usecase's name should be on the next line
				nextIsName = line.trim().equals("[name]");
				
				//if [name] is found, then the usecase has started
				if (nextIsName) inside = true;
			} else {
				//if [steps] found, then the steps are found next line
				if (line.equals("[steps]")){
					insideSteps = true; 
					//If [end] is found
				} else if (line.equals("[end]")){
					//Then append the usecaes to our collection
					usecase.setUrl(url);
					collection.addUseCase(usecase);					
					usecase = new UseCase();					
					inside = false;
					insideSteps = false;
				} else if (insideSteps){
					//If we are inside the usecase, name has been found but no [end]
					//then its a step for the usecase
					usecase.addStep(line);					
				}
			}
		}
		reader.close();
		return collection;
	}

	/**
	 * Tests if this adapter can read the file. For now, only test is that the file ends with ".txt"
	 * @param url url of the file to test
	 * @return true if this adapter can read the file
	 */
	public boolean canRead(URL url) {
		if (url.getFile().toLowerCase().endsWith(".txt")) 
			return true;
		return false;
	}

	public void setOptions(Properties options) {
		this.options = options;		
	}

	public Properties getOptions() {		
		return options;
	}
	

}
