package ucot.ui.gui;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.HashSet;
import java.util.Set;

import javax.swing.JComponent;

import ucot.model.AnalyzeModel;
import ucot.model.Entity;
import ucot.model.Relation;
import ucot.model.RelationType;
import ucot.model.Thing;

public class StructurePanel extends JComponent {

    /**
     * Class to handle "entityBox" wich is drawed into structurepanel, knows 
     * it's location, size and how to paint itself. 
     * 
     * @author ilanliuk
     */
    private class EntityBox {

        private Point location;
        private Dimension size;
        private String name;
        
        public String getName() {
            return name;
        }

        public EntityBox(Point location, String name) {
            
            this.location = location;
            this.size = new Dimension(ENTITYSIZE_X, ENTITYSIZE_Y);
            this.name = name;
            
        }
        
        /**
         * Draws the entity box into given graphics.
         * @param g
         */
        public void paintTo(Graphics g) {

            g.setColor(Color.BLACK);
            
            g.drawRect(location.x, location.y, size.width, size.height);


            int margy = g.getFontMetrics().getHeight() + 5;
            int margx = 10;

            g.drawString(this.name, location.x + margx, location.y + margy);
        
        }
        
        /**
         * returns true if given Point p is inside the box.
         * @param p Point
         * @return 
         */
        public boolean isPointInsideBox(Point p) {
            
            if (location.x < p.x && p.x < (location.x + size.width)) 
                if (location.y < p.y && p.y < (location.y + size.height)) return true;
            
            return false;
        }
        
    }
    
    
    // Size of the entity box
    public static final int ENTITYSIZE_X = 90;
    public static final int ENTITYSIZE_Y = 50;

    private AnalyzeModel model;

    private Thing selectedEntity;

    private Set<EntityBox> entityboxes = new HashSet<EntityBox>();
    
    public StructurePanel() {
        super();
        // TODO Auto-generated constructor stub

        model = null;       
        
        this.addMouseListener( new MouseListener() {
        
            public void mouseExited(MouseEvent arg0) {
                // TODO Auto-generated method stub
                // no need to do enything
            }
        
            public void mouseEntered(MouseEvent arg0) {
                // TODO Auto-generated method stub
                // no need to do enything
            }
        
            public void mouseReleased(MouseEvent arg0) {
                // TODO Auto-generated method stub
                // no need to do enything
            }
        
            public void mousePressed(MouseEvent arg0) {
                // TODO Auto-generated method stub
                // no need to do enything
            }
        
            public void mouseClicked(MouseEvent e) {
                
                // Check if the point where user clicked was 
                // over some entitybox
                Point p = e.getPoint();
                EntityBox target = null;
                
                for (EntityBox eb : entityboxes) {
                    if ( eb.isPointInsideBox(p) ) target = eb;
                }
                if (target != null) {
                    Entity ent = model.getEntityByName(target.getName());
                    System.out.println("Click on:" + ent.getName());
                    
                }
            }
        
        });
    }

    /**
     * method to update the model
     * 
     * @param am
     *            AnalyzeModel
     * @param selectedEntity
     *            wich entity is selected
     */
    public void updateModel(AnalyzeModel am, Thing selectedEntity) {

        if (am != null)
            this.model = am;
        if (selectedEntity != null)
            this.selectedEntity = selectedEntity;

        this.repaint();

    }

    /**
     * Override components paint method. Draws selected entity and entities it
     * links to.
     */
    protected void paintComponent(Graphics g) {

        // strcture can't be drawn if model or selectedEntity is null
        if (this.model != null && this.selectedEntity != null) {

            // clear old entityboxes from set
            entityboxes.clear();
            
            int x = 20;
            int y = 20;

            // get relation from the entity user has selected
            Set<Relation> relfrom = model.getRelationsFrom(selectedEntity);

            // Count how many relations with INFLUENCE type there is
            int count = 0;
            for (Relation rel : relfrom) {
                if (rel.getType().equals(RelationType.INFLUENCE)) count++;
            }
            
            // count height needed for drawing the entities so we can draw 
            // the selected entity into center.
            int dy = 15 + 65 * count;
 
            // mihin viitattavat entiteetit piirretn
            int relationY = y + 15;

            // piirretn pentiteetti puoleen vliin
            entityboxes.add( new EntityBox(new Point(x, y + ((dy - ENTITYSIZE_Y) / 2)), selectedEntity.getName()) );
            
            // piirretn entiteetit joihin "pentiteetti" viittaa
            for (Relation rel : relfrom) {

                // if relations type is other than INFLUENCE 
                // we don't want to show it
                if (rel.getType().equals(RelationType.INFLUENCE)) {

                    entityboxes.add( new EntityBox(new Point(x + 300, relationY), rel.getTo().getName()) );
                    
                    g.drawLine(x + ENTITYSIZE_X, y + dy / 2, x + ENTITYSIZE_X
                            + 80, relationY + (ENTITYSIZE_Y / 2));
                    g.drawLine(x + ENTITYSIZE_X + 80, relationY
                            + (ENTITYSIZE_Y / 2), x + 300, relationY
                            + (ENTITYSIZE_Y / 2));

                    g.drawString(rel.getName(), x + ENTITYSIZE_X + 100,
                            relationY + (ENTITYSIZE_Y / 2) - 2);
                    
                    relationY += 65;
                    
                }
            }
            
            for (EntityBox eb : entityboxes) {
                eb.paintTo(g);
            }

        }
    }
}
