package ucot.ui.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.net.URL;

import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JSplitPane;
import javax.swing.tree.TreePath;

import ucot.core.ControlInterface;
import ucot.heuristic.AbbottsHeuristic;
import ucot.input.SimpleInputAdapter;
import ucot.input.UseCaseCollection;
import ucot.model.AnalyzeModel;
import ucot.model.Entity;
import ucot.model.Item;
import ucot.model.Method;
import ucot.model.Relation;
import ucot.model.Thing;
import ucot.output.Output;
import ucot.parser.ParsedData;
import ucot.parser.SimpleInputParser;
import ucot.ui.UIInterface;
import ucot.ui.gui.tree.AnalyzeModelTree;
import ucot.ui.gui.tree.TreeItem;

/**
 * Simple UI test.
 * 
 * @author ilanliuk
 *
 */
public class TestUI extends JFrame implements UIInterface{
	
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 1519621172685013541L;
	ControlInterface core = null;
	ActionListener menuListener;
	
	AnalyzeModel model;
	
	AnalyzeModelTree tree;
	
	StructurePanel panel;
	
	EntityPropertiesDialog entProperties = new EntityPropertiesDialog(this);
	SelectUseCaseDialog useCaseSelector = new SelectUseCaseDialog(this);
	
	/**
	 * Constuctor
	 * 
	 * this sets up the UI window.
	 */
	public TestUI() {
		
		model = null;
		
		panel = new StructurePanel();
		
		/**
		 * ActionListener for menu events
		 */
		menuListener = new ActionListener() {
			
			/**
			 * here are handeled the menu events
			 */
			public void actionPerformed(ActionEvent e) {
				System.out.println("Action: " + e.getActionCommand());
				
				//
				// check what action just happened
				//
				if ("open".equals(e.getActionCommand())) {
					System.out.println("User wants to load file.\n");
					// show file open dialog here..
					
					// now uses dummy method to load analyzemodel
					model = getAnalyzemodel();
					
					panel.updateModel(model, null);
					tree.updateAnalyzeModel(model);										
				}
				
				if ("exit".equals(e.getActionCommand())) {
					System.out
					.println("User wants to quit program.\nprogram closing..");
					System.exit(0);
				}
				
				if ("settings".equals(e.getActionCommand())) {
					System.out
					.println("User wants to change program settings.\n");
					// Show settings dialog here..
				}
				
			}
			
		};
		
		this.setLayout(new BorderLayout());
		
		// create the tree form AnalyzeModel
		tree = new AnalyzeModelTree();
		tree.setMinimumSize(new Dimension(250, 400));
		tree.addMouseListener( new MouseListener() {
			
			public void mouseExited(MouseEvent arg0) {
				// nothing to do
			}
			
			public void mouseEntered(MouseEvent arg0) {
				// nothing to do
			}
			
			public void mouseReleased(MouseEvent arg0) {
				// nothing to do
			}
			
			public void mousePressed(MouseEvent arg0) {
				// nothing to do
			}
			
			public void mouseClicked(MouseEvent e) {
				
				// See if we cliked to any node in the tree.
				final TreePath selectionPath = tree.getPathForLocation(e.getX(), e.getY());
				
				// if user clicked second mouse button
				// (MouseEvent.BUTTON2 is scroll)
				if (e.getButton() == MouseEvent.BUTTON3) {
					
					System.out.println("Tree: *RIGHT CLICK*");
					
					// Set selection right.
					tree.setSelectionPath(selectionPath);
					
					
					// Creating popup if something is selected.				
					if (selectionPath != null){
						final Object selected = ((TreeItem) selectionPath.getLastPathComponent()).getData();
						JPopupMenu menu = new JPopupMenu();
						
						//Choices:
                        JMenuItem createEntity = new JMenuItem("New entity..");
						JMenuItem modify = new JMenuItem("Modify..");
						JMenuItem deleteEntity = new JMenuItem("Delete entity");
						JMenuItem deleteRelation = new JMenuItem("Delete relation");
						
                        // Add listener for createEntity event.
                        createEntity.addActionListener(new ActionListener() {
                            public void actionPerformed(ActionEvent e) {
                                System.out.println("Popup createEntity");
                                
                                // Add entity called "new entity" to analyzeModel and
                                // then open modifying dialog with it..
                                Entity ent = new Entity("new entity");
                                model.addItem(ent);
                                entProperties.modifyEntityProperties(ent, model);
                                
                            }
                        
                        });
                        
                        
                        
						// Add listener for modify event.
						modify.addActionListener(new ActionListener(){
							public void actionPerformed(ActionEvent e) {
								System.out.println("Popup modify: " + selected.toString());
								//entProperties.setVisible(true);
								if (selected instanceof Item) {
									entProperties.modifyEntityProperties((Entity) selected , model);
								}
							}
							
						});
						

						// Add choices to menu.
						if (selected instanceof Entity) menu.add(modify);
												 

						
						if (selected instanceof Entity && tree.getEntityFromPath(selectionPath) == null){
							// Add listener for delete event.						
							deleteEntity.addActionListener(new ActionListener(){
								public void actionPerformed(ActionEvent e) {
									System.out.println("Popup delete");								
									if (null != selected && (selected instanceof Item)){
										
										model.removeItem((Item) selected);
									}
								}
								
							});
							menu.add(deleteEntity);
						}

						if (selected instanceof Entity && tree.getEntityFromPath(selectionPath) != null && tree.getMethodFromPath(selectionPath) != null){
							// Add listener for delete event.						
							deleteRelation.addActionListener(new ActionListener(){
								public void actionPerformed(ActionEvent e) {
									System.out.println("Popup delete relation");								
									if (null != selected && (selected instanceof Item)){
										
										Entity ent = tree.getEntityFromPath(selectionPath);
										Method method = tree.getMethodFromPath(selectionPath);
										
										Relation rel = model.getRelation(ent,method,(Thing) selected);
										if (null != rel){										
											model.removeRelation(rel);
										}
									}

								}
								
							});
							menu.add(deleteRelation);
						}

						

						// Add choices to menu.
                        menu.add(createEntity);
                        if (selected instanceof Entity) menu.add(modify); 

						
						// Show popup menu relative to the components coordinates.
						// This shows popup exactly where user pressed.
						menu.show(e.getComponent(),e.getX(),e.getY());
					}
					
				}
				
			}
			
		} ); 
		
		
		tree.setStructurePanel(panel);
		
		this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		JSplitPane sp = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, tree, panel);
		this.getContentPane().add(sp, BorderLayout.CENTER);
		
		this.setJMenuBar(createMenu());
		
		this.setTitle("TestUI");
		this.setVisible(true);
		
		// set default size for window
		this.getContentPane().setPreferredSize(new Dimension(900, 550));
		this.pack();
		
	}
	
	/**
	 * method to create menubar for UI
	 * @return JMenuBar
	 */
	public JMenuBar createMenu() {
		
		/**
		 * Menu looks like:
		 * 
		 * +------+----------+--------
		 * | File | Program  |
		 * ===========================
		 * | Open | Settings |
		 * +------+----------+
		 * | Exit |
		 * +------+
		 * 
		 */
		
		JMenuBar bar = new JMenuBar();
		
		JMenu m = new JMenu("File");
		
		JMenuItem i = new JMenuItem("Open");
		i.addActionListener(menuListener);
		i.setActionCommand("open");
		m.add(i);
		
		i = new JMenuItem("Exit");
		i.addActionListener(menuListener);
		i.setActionCommand("exit");
		m.add(i);
		
		bar.add(m);
		
		m = new JMenu("Program");
		
		i = new JMenuItem("Settings");
		i.addActionListener(menuListener);
		i.setActionCommand("settings");
		m.add(i);
		
		bar.add(m);
		
		return bar;
		
	}
	
	
	
	/**
	 * returns analyze model from esimfile.txt
	 * @return AnalyzeModel
	 */
	private AnalyzeModel getAnalyzemodel() {
		
		// Heuristic
		AbbottsHeuristic ab = new AbbottsHeuristic();
		Output output = new Output();
		AnalyzeModel model = new AnalyzeModel();
		
		// Open fileDialog 
		JFileChooser fc = new JFileChooser(".");
		int returnVal = fc.showOpenDialog(this);
		
		if (returnVal == JFileChooser.APPROVE_OPTION) {
			
			System.out.println("Choosed file> "
					+ fc.getSelectedFile().getName());
			
			try {
				
				URL url = fc.getSelectedFile().toURL();
				
				// File f = new File("esimfile.txt");
				// URL url = f.toURL();
				
                System.out.println("URL: " + url.getProtocol() + " / " + url.getHost() + " / " + url.getPort()  + " / " + url.getFile());
                
           
                
				SimpleInputAdapter sia = new SimpleInputAdapter();
				SimpleInputParser sip = new SimpleInputParser();
				
				UseCaseCollection collection = sia.read(url);
				
				for (int i = 0; i < collection.getUseCaseCount(); i++) {
					System.out.println(collection.getUseCase(i));
					
				}
				
				if (collection.getUseCaseCount() > 0) {
					// Ask user which use case he or she desires to load.
					ParsedData data = sip.parse(useCaseSelector.selectUseCase(collection));					
					
					System.out.println(data);
					
					// Create AnalyzeModel from the ParsedData
					ab.doHeuristic(data,model);
				} else {
					throw new Exception("No available use cases or the file format is not supported.");
				}
				
			} catch (Exception e) {
				System.out.println(e.getMessage());
				System.err.println(e.getStackTrace().toString());
			}
			
		}
		return model;
		
	}
	
	
	
	
	
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		// TODO Auto-generated method stub
		
		TestUI ui = new TestUI();
		
	}

	public void exportDone() {
		System.out.println("Core told us that export is done");		
	}

	public void analyzeModelLoaded() {
		System.out.println("Core told us that analyzemodel is loaded");
		
	}

	public void useCasesLoaded() {
		System.out.println("Core told us that usecases are loaded");		
	}

	public void useCaseAdded() {
		System.out.println("Core told us that usecase was added to analyzemodel");		
	}

	public void setControlInterface(ControlInterface a) {
		core = a;		
	}
	
}
