package ucot.ui.gui.tree;

import java.awt.Cursor;
import java.awt.Point;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.dnd.DnDConstants;
import java.awt.dnd.DragGestureEvent;
import java.awt.dnd.DragGestureListener;
import java.awt.dnd.DragSource;
import java.awt.dnd.DropTarget;
import java.awt.dnd.DropTargetDragEvent;
import java.awt.dnd.DropTargetDropEvent;
import java.awt.dnd.DropTargetEvent;
import java.awt.dnd.DropTargetListener;

import javax.swing.JTree;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;

import ucot.model.AnalyzeModel;
import ucot.model.Entity;
import ucot.model.Method;
import ucot.model.Relation;
import ucot.ui.gui.StructurePanel;

public class AnalyzeModelTree extends JTree implements DragGestureListener, DropTargetListener {
	
	// TreeSelectionListener for reacting when user selects node
	TreeSelectionListener treeSelectionListener;
	
	// who is notified when user selects a node..
	StructurePanel structurePanel = null;
	
	DragSource dragSource;
	DropTargetListener dtListener;
	DropTarget dropTarget;
	AnalyzeTreeModel atm = new AnalyzeTreeModel();
	AnalyzeModel analyzemodel = null;
	public AnalyzeModelTree() {
		
		//this.setDragEnabled(true);
		this.dragSource = DragSource.getDefaultDragSource();
		
		
		this.dragSource.createDefaultDragGestureRecognizer(
				this, DnDConstants.ACTION_MOVE, this );
		
		
		this.setModel(atm);
		this.dtListener = this;
		
		// component, ops, listener, accepting
		this.dropTarget = new DropTarget(this,
				this.acceptableActions,
				this.dtListener,
				true);
		
		// create the TreeSelectionListener and override selection event..
		treeSelectionListener = new TreeSelectionListener() {
			
			public void valueChanged(TreeSelectionEvent e) {
				
				TreeItem item = (TreeItem) e.getPath().getLastPathComponent();
				
				// if structurepanel is assigned..
				if (structurePanel == null) return;
				if (item == null) return;
				
				Object data = item.getData();
				// if the node selected is not an itemNode there is no reason to continue..
				if (! ( data instanceof Entity )) return;
				
				
				// if selected node is entity, notify the structurePanel 
				// that selectection has changed
				if ( ((Entity) data).getTypeName().equals("entity"))
					structurePanel.updateModel(null, (Entity) data);				
			}
			
		};
		this.addTreeSelectionListener( treeSelectionListener );
		
	}
	
	private int acceptableActions = DnDConstants.ACTION_COPY;
	
	public void dragGestureRecognized(DragGestureEvent dragEvent) {
		TreePath path = getSelectionModel().getSelectionPath();
		if (null != path){
			TreeItem treeItem = (TreeItem) path.getLastPathComponent();
			Object data = treeItem.getData();
			if (data instanceof Entity){
				dragEvent.startDrag(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR), 
						new DataTransfer(data));
			}
			
		}
		
	}
	
	public void dragEnter(DropTargetDragEvent arg0) {
		System.out.println(arg0);
	}
	
	public void dragExit(DropTargetEvent arg0) {
		System.out.println(arg0);
		
	}
	
	public void dragOver(DropTargetDragEvent arg0) {
		
		
	}
	
	public void drop(DropTargetDropEvent dtde){
		
		System.out.println("DROP");
		//The Drag operation has terminated with a Drop on this DropTarget
		Transferable transferable = dtde.getTransferable();
		System.out.println(transferable);

		try{
			
			Entity entity = (Entity) transferable.getTransferData(DataFlavor.stringFlavor);			
			
			
			Point loc = dtde.getLocation();
			
			TreePath destinationPath = getPathForLocation(loc.x, loc.y);
			if (destinationPath != null) {
				Entity parentEntity = getEntityFromPath(destinationPath);
				Method relationMethod = getMethodFromPath(destinationPath);

				if (parentEntity != null && relationMethod != null){
					
					analyzemodel.addRelation(new Relation(parentEntity,relationMethod.getName(),entity));
					
					dtde.dropComplete(true);
				}
			}
		}catch(Exception e){
			e.printStackTrace();
		}
		System.out.println("END OF DROP\n");
		
		
	}
	
	public Method getMethodFromPath(TreePath path){
		if (path != null) {
			for(int i=path.getPathCount()-1; i>0; i--){
				TreeItem ti = (TreeItem) path.getPathComponent(i);
				if (ti.getData() instanceof Method){
					return (Method) ti.getData();
				}
			}
		}
		return null;
	}
	
	public Entity getEntityFromPath(TreePath path){
		if (path != null) {
			// We skip the first.
			for(int i=path.getPathCount()-2; i>0; i--){
				TreeItem ti = (TreeItem) path.getPathComponent(i);
				if (ti.getData() instanceof Entity){
					return (Entity) ti.getData();
				}
			}
		}
		return null;
	}

	
	public void printNodeContents(TreeNode parent)
	{
		System.out.println("Node is " + parent);
		for(int i = 0 ; i<parent.getChildCount(); i++){
			System.out.println(parent.getChildAt(i));
		}
	}
	public void dropActionChanged(DropTargetDragEvent arg0) {
		System.out.println(arg0);
		
	}
	
	/**
	 * method to give the tree a new analyzemodel if model is updated.
	 * @param model updated model
	 */
	public void updateAnalyzeModel(AnalyzeModel analyzemodel) {
		if (atm == null) return;
		this.analyzemodel = analyzemodel; 
		atm.setAnalyzeModel(analyzemodel);	
	}
	
	public void setStructurePanel(StructurePanel structurePanel) {
		this.structurePanel = structurePanel;
	}
	
	
}
