package ucot.util;

import java.util.Collections;

/**
 * QuickSort algorithm for sorting integer and string arrays.
 * @author tujupien
 */
public class QuickSort {
	/**
	 * Static method for sorting integer arrays.
	 * @param array Array of integers to be sorted.
	 * @param startIndex Start index of the section to be sorted.
	 * @param stopIndex End index of the section to be sorted.
	 */
    protected static int[] sort(int array[], int startIndex, int stopIndex) {
        int x = startIndex;
        int y = stopIndex;
        
        if ( x >= y ) {
            return array;
        }
        
        int i = array[(x + y) / 2];
        while ( x < y ) {
            while ( x < y && array[x] < i ) {
                x++;
            }
            
            while ( x < y && array[y] >= i ) {
                y--;
            }
            
            if ( x < y ) {
                int temp = array[x];
                array[x] = array[y];
                array[y] = temp;
            }
        }
        
        if ( y < x ) {
            int temp = y;
            y = x;
            x = temp;
        }
        
        array = sort(array, startIndex, x);
        array = sort(array, x == startIndex ? x + 1 : x, stopIndex);
        return array;
    }
    
    /**
     * Method for sorting entire integer arrays with QuickSort algorithm.
     * @param array Integer array to be sorted.
     */
    public static int[] sort(int array[]) {
        array = sort(array, 0, array.length-1);
        return array;
    }
    
    /**
     * Calculates a double value of a string based on the given string's
     * alphabetical value. For example string 'foo' has larger double value
     * than 'bar'.
     * @param str String to be calculated.
     * @return Double value of a string.
     */
    protected static double doubleValue(String str) {
    	double d = 0;
    	for ( int i = 0; i < str.length(); i++ )
    		d += (double)(Integer.valueOf(str.charAt(i)) / (((double)i * 2000) + 1));
    	return d;
    	
    }
    
    /**
     * Method for sorting string arrays case-sensitevely with QuickSort.
     * @param array String array to be sorted.
     * @param startIndex Start index of the section to be sorted.
     * @param stopIndex End index of the section to be sorted.
     */
    protected static String[] sort(String array[], int startIndex, int stopIndex) {
        int x = startIndex;
        int y = stopIndex;
        
        if ( x >= y ) {
            return array;
        }
        
        String i = array[(x + y) / 2];
        while ( x < y ) {
            while ( x < y && doubleValue(array[x]) < doubleValue(i) ) {
                x++;
            }
            
            while ( x < y && doubleValue(array[y]) >= doubleValue(i) ) {
                y--;
            }
            
            if ( x < y ) {
                String temp = array[x];
                array[x] = array[y];
                array[y] = temp;
            }
        }
        
        if ( y < x ) {
            int temp = y;
            y = x;
            x = temp;
        }
        
        array = sort(array, startIndex, x);
        array = sort(array, x == startIndex ? x + 1 : x, stopIndex);
        return array;
    }
    
    /**
     * Method for sorting string arrays case-insensitevely with QuickSort.
     * @param array String array to be sorted.
     * @param startIndex Start index of the section to be sorted.
     * @param stopIndex End index of the section to be sorted.
     */
    protected static String[] sortIgnoreCase(String array[], int startIndex, int stopIndex) {
        int x = startIndex;
        int y = stopIndex;
        
        if ( x >= y ) {
            return array;
        }
        
        String i = array[(x + y) / 2];
        while ( x < y ) {
            while ( x < y && doubleValue(array[x].toLowerCase()) < doubleValue(i.toLowerCase()) ) {
                x++;
            }
            
            while ( x < y && doubleValue(array[y].toLowerCase()) >= doubleValue(i.toLowerCase()) ) {
                y--;
            }
            
            if ( x < y ) {
                String temp = array[x];
                array[x] = array[y];
                array[y] = temp;
            }
        }
        
        if ( y < x ) {
            int temp = y;
            y = x;
            x = temp;
        }
        
        array = sortIgnoreCase(array, startIndex, x);
        array = sortIgnoreCase(array, x == startIndex ? x + 1 : x, stopIndex);
        return array;
    }
    
    /**
     * Method for sorting string arrays with QuickSort.
     * @param array String array to be sorted.
     * @param ignoreCase Boolean value which indicates wether the string array
     *   is to be sorted case-sensitevely or case-insensitively.
     */
    public static String[] sort(String array[], boolean ignoreCase) {
    	if ( ignoreCase )
    		array = sortIgnoreCase(array, 0, array.length-1);
    	else
    		array = sort(array, 0, array.length-1);
    	return array;
    }
}
